package com.plannaplan.repositories;

import static org.junit.Assert.assertTrue;

import java.util.Optional;

import com.plannaplan.entities.User;
import com.plannaplan.services.UserService;
import com.plannaplan.types.UserRoles;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

@RunWith(SpringRunner.class)
@SpringBootTest
@ContextConfiguration
public class UserRepositoryTest {
    
    @Autowired
    private UserService userService;

    @Autowired
    private UserRepository userRepository;

    @Test
    public void shouldReturnByAuthorityWithGivenEmail(){
        final String email = "shouldReturnByAuthorityWithGivenEmail@UserRepository.Test";
        final String usosId = "45678";
        final User user = this.userService.save(new User("shouldReturnByAuthority", "WithGivenEmail", email, usosId, UserRoles.TEST_USER));

        final Optional<User> response = this.userRepository.getByAuthority(email);
        assertTrue(response.get().getEmail().equals(email));
        assertTrue(response.get().getUsosId().equals(usosId));
        assertTrue(response.get().getId().equals(user.getId())); 
    }

    @Test
    public void shouldReturnByAuthorityWithUsosId(){
        final String email = "shouldReturnByAuthorityWithUsosId@UserRepository.Test";
        final String usosId = "45678";
        final User user = this.userService.save(new User("shouldReturnByAuthority", "WithGivenEmail", email, usosId, UserRoles.TEST_USER));

        final Optional<User> response = this.userRepository.getByAuthority(usosId);
        assertTrue(response.get().getEmail().equals(email)); 
        assertTrue(response.get().getUsosId().equals(usosId));
        assertTrue(response.get().getId().equals(user.getId())); 
    }
}
